import React, { useCallback, useState } from "react";
import pick from "lodash/pick";

import {
  acceptProposal,
  announceProposal,
  dislike,
  edit,
  hide,
  like,
  loadPosts,
  rejectProposal,
  rejectProposalByChairman,
} from "actions/posts";
import { ban } from "actions/users";
import ErrorMessage from "components/ErrorMessage";
import ModalConfirm from "components/modals/ModalConfirm";
import PostEditModal from "components/posts/PostEditModal";
import PostList from "components/posts/PostList";
import { useActionLoading, useItemActionConfirm } from "hooks";
import { AuthStore, PostStore } from "stores";

const PostsContainer = ({ className }) => {
  const [postToEdit, setPostToEdit] = useState(null);

  const [
    userToBan,
    setUserToBan,
    onBanUserConfirm,
    onBanUserCancel,
  ] = useItemActionConfirm(ban);
  const [
    postToHide,
    setPostToHide,
    onPostHideConfirm,
    onPostHideCancel,
  ] = useItemActionConfirm(hide);
  const [
    postToAnnounce,
    setPostToAnnounce,
    onAnnounceConfirm,
    onAnnounceCancel,
  ] = useItemActionConfirm(announceProposal);
  const [
    postToAccept,
    setPostToAccept,
    onAcceptConfirm,
    onAcceptCancel,
  ] = useItemActionConfirm(acceptProposal);
  const [
    postToReject,
    setPostToReject,
    onRejectConfirm,
    onRejectCancel,
  ] = useItemActionConfirm(rejectProposal);
  const [
    postToRejectByChairman,
    setPostToRejectByChairman,
    onRejectByChairmanConfirm,
    onRejectByChairmanCancel,
  ] = useItemActionConfirm(rejectProposalByChairman);

  const { isAuthenticated, user } = AuthStore.useState();
  const { window, items } = PostStore.useState((state) =>
    pick(state, ["window", "items"])
  );
  const showingArchivedOnly = PostStore.useState(
    (state) => state.filters.flags === "archived"
  );

  const banningUser = useActionLoading(ban, userToBan);
  const hidingPost = useActionLoading(hide, postToHide);
  const announcingProposal = useActionLoading(announceProposal, postToAnnounce);
  const acceptingProposal = useActionLoading(acceptProposal, postToAccept);
  const rejectingProposal = useActionLoading(rejectProposal, postToReject);
  const rejectingProposalByChairman = useActionLoading(
    rejectProposalByChairman,
    postToRejectByChairman
  );

  const { 2: loadResult } = loadPosts.useWatch();

  const confirmEdit = useCallback(
    async (newContent) => {
      if (postToEdit && newContent) {
        await edit.run({ post: postToEdit, newContent });
        setPostToEdit(null);
      }
    },
    [postToEdit, setPostToEdit]
  );

  const cancelEdit = useCallback(() => {
    setPostToEdit(null);
  }, [setPostToEdit]);

  /**
   * Ban a post's author.
   * @param {CF2021.Post} post
   */
  const onBanUser = (post) => {
    setUserToBan(post.author);
  };

  /**
   * Mark down user saw this post already.
   * @param {CF2021.Post} post
   */
  const markSeen = (post) => {
    PostStore.update((state) => {
      state.items[post.id].seen = true;
    });
  };

  const sliceStart = (window.page - 1) * window.perPage;
  const sliceEnd = window.page * window.perPage;

  return (
    <>
      {loadResult && loadResult.error && (
        <ErrorMessage>
          Příspěvky se nepodařilo načíst: {loadResult.message}
        </ErrorMessage>
      )}
      <PostList
        items={window.items
          .slice(sliceStart, sliceEnd)
          .map((postId) => items[postId])}
        canThumb={isAuthenticated}
        canRunActions={isAuthenticated && user.role === "chairman"}
        onLike={like.run}
        onDislike={dislike.run}
        onSeen={markSeen}
        className={className}
        dimArchived={!showingArchivedOnly}
        onHide={setPostToHide}
        onBanUser={onBanUser}
        onEdit={setPostToEdit}
        onAnnounceProcedureProposal={setPostToAnnounce}
        onAcceptProcedureProposal={setPostToAccept}
        onRejectProcedureProposal={setPostToReject}
        onRejectProcedureProposalByChairman={setPostToRejectByChairman}
      />
      <ModalConfirm
        isOpen={!!userToBan}
        onConfirm={onBanUserConfirm}
        onCancel={onBanUserCancel}
        confirming={banningUser}
        title={`Zablokovat uživatele ${userToBan ? userToBan.name : null}?`}
        yesActionLabel="Zablokovat"
      >
        Uživatel <strong>{userToBan ? userToBan.name : null}</strong> bude
        zablokován a nebude dále moci vkládat nové příspěvky. Opravdu to chcete?
      </ModalConfirm>
      <ModalConfirm
        isOpen={!!postToHide}
        onConfirm={onPostHideConfirm}
        onCancel={onPostHideCancel}
        confirming={hidingPost}
        title="Skrýt příspěvek?"
        yesActionLabel="Potvrdit"
      >
        Příspěvek se skryje a uživatelé ho neuvidí. Opravdu to chcete?
      </ModalConfirm>
      <ModalConfirm
        isOpen={!!postToAnnounce}
        onConfirm={onAnnounceConfirm}
        onCancel={onAnnounceCancel}
        confirming={announcingProposal}
        title="Vyhlásit procedurální návrh?"
        yesActionLabel="Vyhlásit návrh"
      >
        Procedurální návrh bude <strong>vyhlášen</strong>. Opravdu to chcete?
      </ModalConfirm>
      <ModalConfirm
        isOpen={!!postToAccept}
        onConfirm={onAcceptConfirm}
        onCancel={onAcceptCancel}
        confirming={acceptingProposal}
        title="Schválit procedurální návrh?"
        yesActionLabel="Schválit návrh"
      >
        Procedurální návrh bude <strong>schválen</strong>. Opravdu to chcete?
      </ModalConfirm>
      <ModalConfirm
        isOpen={!!postToReject}
        onConfirm={onRejectConfirm}
        onCancel={onRejectCancel}
        confirming={rejectingProposal}
        title="Zamítnout procedurální návrh?"
        yesActionLabel="Zamítnout návrh"
      >
        Procedurální návrh bude <strong>zamítnut</strong>. Opravdu to chcete?
      </ModalConfirm>
      <ModalConfirm
        isOpen={!!postToRejectByChairman}
        onConfirm={onRejectByChairmanConfirm}
        onCancel={onRejectByChairmanCancel}
        confirming={rejectingProposalByChairman}
        title="Zamítnout procedurální návrh předsedajícícm?"
        yesActionLabel="Zamítnout návrh předsedajícím"
      >
        Procedurální návrh bude <strong>zamítnut předsedajícím</strong>. Opravdu
        to chcete?
      </ModalConfirm>
      {postToEdit && (
        <PostEditModal
          isOpen={true}
          post={postToEdit}
          onConfirm={confirmEdit}
          onCancel={cancelEdit}
        />
      )}
    </>
  );
};

export default PostsContainer;
